/***********************************************************************************
  Filename: 	per_test.c

  Description:  This application functions as a packet error rate (PER) tester.
  One node is set up as transmitter and the other as receiver. The role and
  configuration parameters for the PER test of the node is chosen on initalisation
  by navigating the joystick and confirm the choices with S1.

  The configuration parameters are channel, burst size and tx power. Push S1 to
  enter the menu. Then the configuration parameters are set by pressing
  joystick to right or left (increase/decrease value) and confirm with S1.

  After configuration of both the receiver and transmitter, the PER test is
  started by pressing joystick up on the transmitter. By pressing joystick up
  again the test is stopped.

***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "hal_int.h"
#include "hal_timer_32k.h"
#include "hal_board.h"
#include "hal_rf.h"
#include "hal_assert.h"
#include "basic_rf.h"
#include "application.h"
#include "CC2530_UART.h"
#include "uart_queue.h"
#include "hal_rf_util.h"
#include "command.h"

/***********************************************************************************
* LOCAL VARIABLES
*/
basicRfCfg_t g_basicRfConfig;
uint16 g_deviceAddr;
uint8 g_txPower;

static app_Packet_t txPacket;
static app_Packet_t rxPacket;

/***********************************************************************************
* @fn          appReceiver
*
* @brief       Application code for the receiver mode. Puts MCU in endless loop
*
* @param       g_basicRfConfig - file scope variable. Basic RF configuration data
*              rxPacket - file scope variable of type perTestPacket_t
*
* @return      none
*/
static void appReceiver()
{
    uint8 i;
    if(basicRfReceive((uint8*)&rxPacket, sizeof(app_Packet_t), NULL)>0) 
    {
        for(i = 0; i < rxPacket.size; i++)
        {
            UartQueue_Tx_EnQueue(rxPacket.data[i]);
        }
    }
}

/***********************************************************************************
* @fn          appTransmitter
*
* @brief       Application code for the transmitter mode. Puts MCU in endless
*              loop
*
* @param       g_basicRfConfig - file scope variable. Basic RF configuration data
*              txPacket - file scope variable of type perTestPacket_t
*              appState - file scope variable. Holds application state
*              appStarted - file scope variable. Controls start and stop of
*                           transmission
*
* @return      none
*/
static void appTransmitter()
{
	uint8 rxChar;
	uint8 txPacketSize;

    txPacketSize = 0;
    do
    {
        rxChar = UartQueue_Rx_DeQueue();
        txPacket.data[txPacketSize++] = rxChar;
        if(txPacketSize == APP_MAX_PAYLOAD_LENGTH)
        {
            break;
        }
    }while(FALSE == UartQueue_Rx_Is_Empty());

    txPacket.size = txPacketSize;
    basicRfSendPacket(g_deviceAddr, (uint8*)&txPacket, txPacketSize + 1);
}

#define COORDINATOR     1

static inline void rf_init_default(void)
{
    // Config basicRF
    g_basicRfConfig.panId = PAN_ID;
    g_basicRfConfig.ackRequest = FALSE;

    // Set channel
    g_basicRfConfig.channel = APP_DEFAULT_CHANNEL;

    // Initialize BasicRF
#ifdef COORDINATOR
    g_basicRfConfig.myAddr = SERVER_ADDR;
    g_deviceAddr           = DEVICE_ADDR;
#else
    g_basicRfConfig.myAddr = DEVICE_ADDR;
    g_deviceAddr           = SERVER_ADDR;
#endif

#if INCLUDE_PA==2591 || INCLUDE_PA==2592
//    g_txPower = HAL_RF_TXPOWER_0_DBM;
//    g_txPower = HAL_RF_TXPOWER_13_DBM;
//    g_txPower = HAL_RF_TXPOWER_16_DBM;
//    g_txPower = HAL_RF_TXPOWER_18_DBM;
    g_txPower = HAL_RF_TXPOWER_20_DBM;
#else
//    g_txPower = HAL_RF_TXPOWER_MIN_3_DBM;
//    g_txPower = HAL_RF_TXPOWER_0_DBM;
    g_txPower = HAL_RF_TXPOWER_4_DBM;
#endif
}

static inline void rf_init_power(void)
{
    int txTmpPower = g_txPower;

    halRfSetTxPower(txTmpPower);

#ifdef INCLUDE_PA
    // Select gain (for modules with CC2590/91 only)
    halRfSetGain(HAL_RF_GAIN_HIGH);
//    halRfSetGain(HAL_RF_GAIN_LOW);
#endif
}

static inline void initialize_rf_common(void)
{
    while(basicRfInit(&g_basicRfConfig)==FAILED)
    {
        Uart_Send_String(g_UartSerialDev, "[!!ERROR!!] basicRfInit\n");
        halMcuWaitMs(100);
    }
    rf_init_power();
    halSetRxNormalMode();
    basicRfReceiveOn();
}

/***********************************************************************************
* @fn          main
*
* @brief       This is the main entry of the "PER test" application.
*
* @param       g_basicRfConfig - file scope variable. Basic RF configuration data
*              appState - file scope variable. Holds application state
*              appStarted - file scope variable. Used to control start and stop of
*              transmitter application.
*
* @return      none
*/

uint8   Uart0_Tx_End = 1;

void main (void)
{
    uint8 cmdPinData = 0, cmdPinData_old = 0;

    // Initialise board peripherals
    halMcuInit();

    // UART0 is communication port
    g_UartCommDev = UART_DEV_UART0;
    Uart0_Init();

    // Debug Uart Select
    g_UartSerialDev = UART_DEV_UART1;
    Uart1_Init();

    rf_init_default();

    // RF initialize funcion
    initialize_rf_common();

    MCU_IO_INPUT(0, 0, MCU_IO_PULLDOWN);

    halMcuWaitMs(100);

    while(1)
    {
        cmdPinData = MCU_IO_GET(0, 0);
        if(cmdPinData_old != cmdPinData)
        {
            cmdPinData_old = cmdPinData;

            if(0 == cmdPinData)
            {
                initialize_rf_common(); // RF initialize funcion
            }
            else
            {
                basicRfReceiveOff();
            }
        }

        if( Uart0_Tx_End == 1 )
        {
            if(FALSE == UartQueue_Tx_Is_Empty())
            {
                U0DBUF = UartQueue_Tx_DeQueue();
                Uart0_Tx_End = 0;
            }
        }

        if(basicRfPacketIsReady() & !cmdPinData)
        {
            appReceiver();
//            Uart_Send_String(g_UartSerialDev, " appReceiver \n");                                        
        }

        if(FALSE == UartQueue_Rx_Is_Empty())
        {
            if(cmdPinData)
            {
//                Uart_Send_String(g_UartSerialDev, " appCommandProcess \n");                            
                appCommandProcess();
            }
            else
            {
//                Uart_Send_String(g_UartSerialDev, " appTransmitter \n");                
                appTransmitter();
            }
        }
    }

    // Role is undefined. This code should not be reached
}

/***********************************************************************************
  Copyright 2008 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS?WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
***********************************************************************************/
